@@ -1,8 +1,8 @@
 /* ldapexop.c -- a tool for performing well-known extended operations */
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapexop.c,v 1.9.2.8 2010/04/15 22:16:50 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2005-2010 The OpenLDAP Foundation.
+ * Copyright 2005-2015 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -75,7 +75,7 @@ main( int argc, char *argv[] )
 	char		*matcheddn = NULL, *text = NULL, **refs = NULL;
 	LDAPControl **ctrls = NULL;
 	int		id, code;
-	LDAPMessage	*res;
+	LDAPMessage	*res = NULL;
 
 	tool_init( TOOL_EXOP );
 	prog = lutil_progname( "ldapexop", argc, argv );
@@ -130,7 +130,7 @@ main( int argc, char *argv[] )
 		}
 
 	} else if ( strcasecmp( argv[ 0 ], "passwd" ) == 0 ) {
-		fprintf( stderr, "use ldappasswd(1) instead.\n\n", argv[ 0 ] );
+		fprintf( stderr, "use ldappasswd(1) instead.\n\n" );
 		usage();
 		/* TODO? */
 
@@ -213,7 +213,7 @@ main( int argc, char *argv[] )
 		struct timeval	tv;
 
 		if ( tool_check_abandon( ld, id ) ) {
-			return LDAP_CANCELLED;
+			tool_exit( ld, LDAP_CANCELLED );
 		}
 
 		tv.tv_sec = 0;
@@ -247,7 +247,7 @@ main( int argc, char *argv[] )
 		char		*retoid = NULL;
 		struct berval	*retdata = NULL;
 
-		rc = ldap_parse_extended_result( ld, res, &retoid, &retdata, 1 );
+		rc = ldap_parse_extended_result( ld, res, &retoid, &retdata, 0 );
 
 		if ( rc != LDAP_SUCCESS ) {
 			tool_perror( "ldap_parse_extended_result", rc, NULL, NULL, NULL, NULL );
@@ -294,7 +294,7 @@ main( int argc, char *argv[] )
 			printf(_("# extended operation response\n"));
 		}
 
-		rc = ldap_parse_extended_result( ld, res, &retoid, &retdata, 1 );
+		rc = ldap_parse_extended_result( ld, res, &retoid, &retdata, 0 );
 		if ( rc != LDAP_SUCCESS ) {
 			tool_perror( "ldap_parse_extended_result", rc, NULL, NULL, NULL, NULL );
 			rc = EXIT_FAILURE;
@@ -318,7 +318,8 @@ main( int argc, char *argv[] )
 		}
 	}
 
-	if( verbose || ( code != LDAP_SUCCESS ) || matcheddn || text || refs ) {
+	if( verbose || code != LDAP_SUCCESS ||
+		( matcheddn && *matcheddn ) || ( text && *text ) || refs ) {
 		printf( _("Result: %s (%d)\n"), ldap_err2string( code ), code );
 
 		if( text && *text ) {
@@ -348,8 +349,7 @@ main( int argc, char *argv[] )
 
 skip:
 	/* disconnect from server */
-	tool_unbind( ld );
-	tool_destroy();
-
-	return code == LDAP_SUCCESS ? EXIT_SUCCESS : EXIT_FAILURE;
+	if ( res )
+		ldap_msgfree( res );
+	tool_exit( ld, code == LDAP_SUCCESS ? EXIT_SUCCESS : EXIT_FAILURE );
 }
