@@ -1,8 +1,8 @@
 /* config.c - DNS SRV backend configuration file routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-dnssrv/config.c,v 1.16.2.4 2009/01/22 00:01:06 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2009 The OpenLDAP Foundation.
+ * Copyright 2000-2015 The OpenLDAP Foundation.
  * Portions Copyright 2000-2003 Kurt D. Zeilenga.
  * All rights reserved.
  *
@@ -29,6 +29,7 @@
 #include "slap.h"
 #include "proto-dnssrv.h"
 
+#if 0
 int
 dnssrv_back_db_config(
     BackendDB	*be,
@@ -50,3 +51,4 @@ dnssrv_back_db_config(
 	/* no configuration options (yet) */
 	return SLAP_CONF_UNKNOWN;
 }
+#endif
