@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/back-sql.h,v 1.49.2.6 2010/04/13 20:23:42 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2010 The OpenLDAP Foundation.
+ * Copyright 1999-2015 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Mararati.
  * Portions Copyright 2004 Mark Adamson.
@@ -183,7 +183,7 @@ typedef struct {
 	UDWORD		*col_prec;
 	SQLSMALLINT	*col_type;
 	char		**cols;
-	SQLINTEGER	*value_len;
+	SQLLEN		*value_len;
 } BACKSQL_ROW_NTS;
 
 /*
@@ -223,6 +223,22 @@ typedef struct {
  */
 #undef BACKSQL_ARBITRARY_KEY
 
+/*
+ * type used for keys
+ */
+#if defined(HAVE_LONG_LONG) && defined(SQL_C_UBIGINT) && \
+	( defined(HAVE_STRTOULL) || defined(HAVE_STRTOUQ) )
+typedef unsigned long long backsql_key_t;
+#define BACKSQL_C_NUMID	SQL_C_UBIGINT
+#define BACKSQL_IDNUMFMT "%llu"
+#define BACKSQL_STR2ID lutil_atoullx
+#else /* ! HAVE_LONG_LONG || ! SQL_C_UBIGINT */
+typedef unsigned long backsql_key_t;
+#define BACKSQL_C_NUMID	SQL_C_ULONG
+#define BACKSQL_IDNUMFMT "%lu"
+#define BACKSQL_STR2ID lutil_atoulx
+#endif /* ! HAVE_LONG_LONG */
+
 /*
  * define to enable support for syncprov overlay
  */
@@ -264,6 +280,8 @@ typedef struct backsql_api {
 
 	void			*ba_private;
 	struct backsql_api	*ba_next;
+	char **ba_argv;
+	int	ba_argc;
 } backsql_api;
 
 /*
@@ -289,7 +307,7 @@ typedef struct backsql_oc_map_rec {
 	/* flags whether delete_proc is a function (whether back-sql 
 	 * should bind first parameter as output for return code) */
 	int			bom_expect_return;
-	unsigned long		bom_id;
+	backsql_key_t		bom_id;
 	Avlnode			*bom_attrs;
 	AttributeDescription	*bom_create_hint;
 } backsql_oc_map_rec;
@@ -381,11 +399,11 @@ typedef struct backsql_entryID {
 #define BACKSQL_MAX_KEY_LEN	64
 #else /* ! BACKSQL_ARBITRARY_KEY */
 	/* The original numeric key is maintained as default. */
-	unsigned long		eid_id;
-	unsigned long		eid_keyval;
+	backsql_key_t		eid_id;
+	backsql_key_t		eid_keyval;
 #endif /* ! BACKSQL_ARBITRARY_KEY */
 
-	unsigned long		eid_oc_id;
+	backsql_key_t		eid_oc_id;
 	backsql_oc_map_rec	*eid_oc;
 	struct berval		eid_dn;
 	struct berval		eid_ndn;
@@ -494,8 +512,9 @@ typedef struct backsql_info {
 	struct berval	sql_upper_func;
 	struct berval	sql_upper_func_open;
 	struct berval	sql_upper_func_close;
-	BerVarray	sql_concat_func;
 	struct berval	sql_strcast_func;
+	BerVarray	sql_concat_func;
+	char		*sql_concat_patt;
 
 	struct berval	sql_aliasing;
 	struct berval	sql_aliasing_quote;
@@ -517,6 +536,7 @@ typedef struct backsql_info {
 #define BSQLF_FETCH_ALL_OPATTRS		0x0400
 #define	BSQLF_FETCH_ALL_ATTRS		(BSQLF_FETCH_ALL_USERATTRS|BSQLF_FETCH_ALL_OPATTRS)
 #define BSQLF_CHECK_SCHEMA		0x0800
+#define BSQLF_AUTOCOMMIT_ON		0x1000
 
 #define BACKSQL_ISF(si, f) \
 	(((si)->sql_flags & f) == f)
@@ -549,8 +569,11 @@ typedef struct backsql_info {
 	BACKSQL_ISF(si, BSQLF_FETCH_ALL_ATTRS)
 #define BACKSQL_CHECK_SCHEMA(si) \
 	BACKSQL_ISF(si, BSQLF_CHECK_SCHEMA)
+#define BACKSQL_AUTOCOMMIT_ON(si) \
+	BACKSQL_ISF(si, BSQLF_AUTOCOMMIT_ON)
 
 	Entry		*sql_baseObject;
+	char		*sql_base_ob_file;
 #ifdef BACKSQL_ARBITRARY_KEY
 #define BACKSQL_BASEOBJECT_IDSTR	"baseObject"
 #define BACKSQL_BASEOBJECT_KEYVAL	BACKSQL_BASEOBJECT_IDSTR
@@ -596,5 +619,13 @@ typedef struct backsql_info {
 	  || (ct) == SQL_VARBINARY \
 	  || (ct) == SQL_LONGVARBINARY)
 
+#ifdef BACKSQL_ARBITRARY_KEY
+#define BACKSQL_IDFMT "%s"
+#define BACKSQL_IDARG(arg) ((arg).bv_val)
+#else /* ! BACKSQL_ARBITRARY_KEY */
+#define BACKSQL_IDFMT BACKSQL_IDNUMFMT
+#define BACKSQL_IDARG(arg) (arg)
+#endif /* ! BACKSQL_ARBITRARY_KEY */
+
 #endif /* __BACKSQL_H__ */
 
